# -*- coding: utf-8 -*-
#
# Author: Lars B. Rollik <L.B.Rollik@protonmail.com>
# License: BSD 3-Clause
import logging
import os
from datetime import datetime


def get_datestr():
    return datetime.now().strftime("%Y%m%d_%H%M%S")


def make_recording_file_names(
    path=None, package_id="rcc", ext_sep=".", video_ext="h264"
):
    # basepath, _ = os.path.splitext(str(path))
    basepath = str(path)
    dt = get_datestr()
    name_ttl_out = ".".join(["timestamps", "ttl", "out"])
    name_ttl_in = ".".join(["timestamps", "ttl", "in"])

    return {
        "video": ext_sep.join(
            [basepath, dt, package_id, "video", str(video_ext).replace(".", "")]
        ),
        "ttl.out": ext_sep.join([basepath, dt, package_id, name_ttl_out, "csv"]),
        "ttl.in": ext_sep.join([basepath, dt, package_id, name_ttl_in, "csv"]),
        "metadata": ext_sep.join([basepath, dt, package_id, "metadata", "json"]),
    }


def close_file_safe(file_handle):
    try:
        name = file_handle.name
    except BaseException:
        name = "not a file object"

    try:
        # Relevant actions
        file_handle.flush()
        os.fsync(file_handle.fileno())
        file_handle.close()
        file_handle = None
        # Report success
        logging.debug(f"Closed file:\t{name}")
    except BaseException:
        logging.debug(f"FAILED TO CLOSE FILE:{name}")


class DummyFileObject(object):
    def __init__(self, *args, **kwargs):
        pass

    def write(self):
        pass

    def flush(self):
        pass

    def open(self):
        pass

    def close(self):
        pass



import logging

from rich.logging import RichHandler


def log_level_name_to_value(name=None, default=20):
    """Translate log level name to corresponding value. Default is INFO==20"""
    return logging._nameToLevel.get(str(name).upper(), default)


def setup_logging_control(level=None):
    if level is None:
        level = "DEBUG"
    logger = logging.getLogger()

    if not logger.handlers:
        logger.setLevel(getattr(logging, level))

        formatter = logging.Formatter("%(message)s")
        formatter.datefmt = "%Y-%m-%d %H:%M:%S.%f"

        logging_handler = RichHandler(
            level=level,
            enable_link_path=False,
            markup=True,
            rich_tracebacks=True,
            tracebacks_show_locals=True,
        )
        logging_handler.setFormatter(formatter)
        logger.addHandler(logging_handler)
        logging.info(f"Set up logging for rcc with level {level}")



# -*- coding: utf-8 -*-
#
# Author: Lars B. Rollik <L.B.Rollik@protonmail.com>
# License: BSD 3-Clause
import logging
import subprocess
import sys
import time
from multiprocessing import Process
from multiprocessing import Queue
from threading import Thread

import psutil
from PyQt5.QtCore import QThread

from rpi_camera_colony.control.conductor import Conductor
from rpi_camera_colony.control.conductor import parse_args_for_conductor
from rpi_camera_colony.tools.comms import execute_in_commandline


def _get_default_python_path():
    return (
        execute_in_commandline(["which", "python"], return_std=True)
        .stdout.read()
        .decode("utf-8")
        .strip("\n")
    )


class ConductorAsSubprocess(object):
    python_path = None
    conductor_args = None
    conductor_args_list = []
    command = None
    conductor_process = None

    def __init__(
        self,
        conductor_args=dict,
        python_path=None,
    ):
        super(ConductorAsSubprocess, self).__init__()

        self.conductor_args = conductor_args

        if python_path is None:
            self.python_path = _get_default_python_path()
        else:
            self.python_path = str(python_path)

        for k, v in self.conductor_args.items():
            arg = "--" + str(k).replace("_", "-")
            self.conductor_args_list.extend([arg, v])

        self.command = [
            self.python_path,
            "-m",
            "rpi_camera_colony.control.main",
        ] + self.conductor_args_list
        self.conductor_process = subprocess.Popen(
            self.command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        logging.debug(
            f"Launched subprocess for rcc Conductor with pid={self.conductor_process.pid}"
        )

    # def __enter__(self):
    #     return self
    #
    # def __exit__(self, exc_type, exc_val, exc_tb):
    #     self.shutdown()

    def __del__(self):
        self.shutdown()

    def shutdown(self):
        if self.conductor_process is not None:
            process_handle = psutil.Process(pid=self.conductor_process.pid)
            psutil.wait_procs(procs=process_handle.children(), timeout=1)
            process_handle.kill()


class ConductorAsProcess(Process):
    daemon = True
    kill_queue = None
    conductor = None

    def __init__(self, conductor_args=None, kill_queue=None):
        super(ConductorAsProcess, self).__init__()

        self.conductor_args = conductor_args
        self.kill_queue = kill_queue

        self.conductor = Conductor(**self.conductor_args)
        self.conductor.start_acquisition()

    def run(self) -> None:
        while self.kill_queue.empty():
            time.sleep(1)

        self.conductor.stop_acquisition()
        self.conductor.cleanup()
        sys.exit(0)

    def stop(self):
        if self.kill_queue is not None:
            self.kill_queue.put(True)

    def __del__(self):
        if self.conductor is not None:
            self.conductor.stop_acquisition()
            self.conductor.cleanup()
            del self.conductor


if __name__ == "__main__":
    # Minimal example for sandboxing Conductor in separate process
    args_for_conductor = parse_args_for_conductor()
    kill_queue = Queue()  # Send value into this queue to stop the Conductor

    conductor_process = ConductorAsProcess(
        conductor_args=args_for_conductor, kill_queue=kill_queue
    )
    conductor_process.run()

    while True:
        try:
            time.sleep(1)
        except KeyboardInterrupt:
            logging.info("Keyboard interrupt. Cleaning up and exiting.")
            conductor_process.join()
            logging.info("Control processed joined and exiting. Good bye.")



import json
import logging
from glob import glob
from pathlib import Path

import pandas as pd


def __read_json(file=None):
    with open(file, "r") as f:
        data = f.read()
    return json.loads(data)


def read_session_data(session_dir=None):
    """

    File name pattern:
        [session_name].camera_51_blue.20210928_100502.rcc.metadata.json
        [session_name].[camera_id].[dt].rcc.[namespace_id]

    Expected files per camera in acquisition:
        - .rcc.metadata.json
        - .rcc.timestamps_ttl_in.csv    : frame timestamps + input timestamps
        - .rcc.timestamps_ttl_out.csv   : frame timestamps == output timestamps
        - .rcc.video.h264
        - .rcc.video.h264.mp4 [only there is run MSW post acquisition tasks]

    """
    session_dir = Path(session_dir)
    assert session_dir.exists()

    namespace_signature = ".rcc."
    rcc_files_in_dir = glob(str(session_dir / f"*{namespace_signature}*"))

    def get_namespace_identifier(file):
        """Return identifier part. Move to new namespace (underscores replaced with dots) for dict keys."""
        return str(file.split(namespace_signature)[-1].replace("_", "."))

    session_data = {}
    for filepath in rcc_files_in_dir:
        filename = Path(filepath).name
        cam = filename.split(".")[1]
        ftype = get_namespace_identifier(filename)

        if cam not in session_data:
            logging.debug(f"New camera found in session '{filename}': {cam}")
            session_data[cam] = {}
            session_data[cam]["has_h264"] = False
            session_data[cam]["has_mp4"] = False

        # Add data
        if "metadata.json" in ftype:
            session_data[cam][ftype.replace(".json", "")] = __read_json(file=filepath)

        elif ftype.endswith(".csv"):
            csv_data = pd.read_csv(filepath)

            # Remove leading hash and whitespace from column names
            for c in csv_data.columns:
                csv_data = csv_data.rename(columns={c: c.strip("#").strip(" ")})

            session_data[cam][ftype.replace(".csv", "")] = csv_data

        elif ftype == "video.h264":
            session_data[cam]["has_h264"] = True

        elif ftype == "video.h264.mp4":
            session_data[cam]["has_mp4"] = True

    return session_data



import platform
from os import path

from setuptools import find_packages
from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md")) as f:
    long_description = f.read()

with open(path.join(this_directory, "LICENSE")) as f:
    license_text = f.read()

if "arm" in platform.machine():
    rpi_install_requires = ["picamera", "RPi.GPIO"]
else:
    rpi_install_requires = []

setup(
    name="rpi_camera_colony",
    version="0.4.1.dev0",
    description="RPi Camera Colony: Central control for video acquisition with (many) Raspberry Pi cameras",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    packages=find_packages() + ["rpi_camera_colony.tools"],
    url="https://github.com/larsrollik/rpi_camera_colony",
    author="Lars B. Rollik",
    author_email="L.B.Rollik@protonmail.com",
    license=license_text,
    install_requires=["pyzmq", "tornado", "tqdm", "configobj", "numpy", "rich"]
    + rpi_install_requires,
    extras_require={
        "dev": [
            "black",
            "pytest-cov",
            "pytest",
            "gitpython",
            "coverage>=5.0.3",
            "bump2version",
            "pre-commit",
            "flake8",
        ],
        "rpi": rpi_install_requires,
    },
    zip_safe=False,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "rcc_conductor = rpi_camera_colony.control.main:main",
            "rcc_acquisition = rpi_camera_colony.acquisition.__main__:main",
        ],
    },
)



# -*- coding: utf-8 -*-
#
# Author: Lars B. Rollik <L.B.Rollik@protonmail.com>
# License: BSD 3-Clause
import logging

from rpi_camera_colony.tools.readers import read_session_data

__version__ = "0.4.1.dev0"
__author__ = "Lars B. Rollik"
